#!/usr/bin/python

#coastercounter.py
#Assignment Four: Python Moderately Complex Assignment - Coaster Counter(for rollercoaster enthusiasts)
#Author: Nicholas Piccillo

#This program tracks rollercoasters for multiple users and: 
#provides a summary of coasters by park name
#provides a total number of coasters recorded
#counts down to coaster count milestones (50, 100, 200, 300, 400, 500, 1000, 1000+)

#Coaster information is saved to and read from a file in json format

#This program accepts a single command line argument for testing purposes

#Usage:
#python coastercounter.py [test data path]

#import modules
import sys
import os
import json

#main Function
def main():
    #variable declarations
    filepath = "./coasterdata.json"
    global data
    global username
    
    #test data
    if (len(sys.argv)>1):
        filepath = sys.argv[1]
        
    #load data from file if it exists
    print ("Welcome to Coaster Counter!\nChecking for data...\n")
    if os.path.isfile(filepath):
        try:
            file = open(filepath,"r")
            if file.read(1): #check for empty file
                file.seek(0)
                data = json.load(file)
            else:
                data = {}
            file.close()
        except Exception as e:
            print ('Data file could not be read : %s\n' % e)
            sys.exit()
    else:
        data = {}
        
    #prompt for username
    username = ""
    while username == "":
        username = input('Please enter your name: ')
        if username == "":
            print ("Username required. Please enter a name!")
        else:
            if username in data:
                print("Welcome back %s!\n" % username)
            else:
                print("Hello %s!\n" % username)
                data[username] = dict() #add user
    #end while
        
    #main loop - prompt for user selection
    flag = True
    while flag == True:
        print ("Would you like to (a)dd a coaster, (l)ist the coasters, (s)ave and exit, or e(x)it without saving?\n")
        selection = input("Please select (a/l/s/x): ")
        
        #add a coaster
        if selection == "A" or selection == "a":
            parkname = input ("Please enter the name of the park where the coaster is located: ")
            coastername = input ("Please enter the name of the coaster from %s: " % parkname)
            useryn = input("Would you like to add %s from %s to your coaster count? (y/n): " % (coastername, parkname))
            if useryn == "y" or useryn == "Y":
                if parkname in data[username]:
                    data[username][parkname].append(coastername)
                else:
                    data[username][parkname] = list()#add new park
                    data[username][parkname].append(coastername)
                print ("Coaster added!\n")
            elif useryn == "n" or useryn == "N":
                pass
            else:
                print("Invalid Selection\n")
                
        #list all coasters for user
        elif selection == "L" or selection == "l":
            for park in data[username]:
                print ("\n%s:" % park)
                for coaster in data[username][park]:
                    print ("\t%s" % coaster)
            coastercount()#call coastercount function for total and milestone
                
        #save and exit
        elif selection == "S" or selection == "s":
            flag = False
            try:
                print("Saving coaster data...")
                file = open(filepath,"w+")
                json.dump(data, file, separators=(',', ':'))
                file.close()
            except Exception as e:
                print('File could not be saved. : %s\n' % e)
                sys.exit()
            print("Save successful.\nThank you for using Coaster Counter!")
            
        #exit without saving
        elif selection == "X" or selection == "x":
            flag = False
            print ("Exiting without saving.\nThank you for using Coaster Counter!")
            
        #invalid selection
        else:
            print("Invalid selection. Please enter a, l, s, or x")
    #end while

#coastercount function
def coastercount():
    #variable declarations
    count = 0
    
    #total coasters from all parks
    for park in data[username]:
        count = count + len(data[username][park])

    #determine next milestone
    if count < 50:
        milestone = 50
    elif count < 100:
        milestone = 100
    elif count < 200:
        milestone = 200
    elif count < 300:
        milestone = 300
    elif count < 400:
        milestone = 400
    elif count < 500:
        milestone = 500
    elif count < 600:
        milestone = 600
    elif count < 700:
        milestone = 700
    elif count < 800:
        milestone = 800
    elif count < 900:
        milestone = 900
    elif count < 1000:
        milestone = 1000
    else:
        milestone = 4639
    togo = milestone - count

    #display total and coasters remaining to next milestone
    print ("\nIt looks like you've experienced %d rollercoasters so far.\n" % count)
    if count < 1000:
        print ("Only %d coasters to go until you reach %d!\n" % (togo, milestone))
    else:
        print ("Wow! That's a lot of rollercoasters!\n")
        print ("There are %d operating rollercoasters in the world (as of 2018)...\njust %d to go!\n" % (milestone, togo))

#Call main function
if __name__ == '__main__':
    main()
